# Generated by Django 3.2.9 on 2021-11-21 15:06

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("wagtail_localize", "0015_translationcontext_field_path"),
        ("wagtail_localize_test", "0018_structblocks_with_choosers"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomTranslationData",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("custom_text_field", models.CharField(max_length=255)),
                (
                    "translation_source",
                    models.ForeignKey(
                        editable=False,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="wagtail_localize.translationsource",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="CustomButSimpleTranslationData",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("notes", models.CharField(max_length=255)),
            ],
        ),
    ]
