# Generated by Django 3.2.4 on 2021-06-29 15:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtail_localize_test", "0014_auto_20210217_0924"),
    ]

    operations = [
        migrations.AddField(
            model_name="testsnippet",
            name="small_charfield",
            field=models.CharField(blank=True, max_length=10),
        ),
    ]
