# Generated by Django 3.1.2 on 2020-10-20 14:10

from django.db import migrations, models
import django.db.models.deletion
import wagtail.core.blocks
import wagtail.core.fields
import wagtail.documents.blocks
import wagtail.images.blocks
import wagtail.snippets.blocks
import wagtail_localize.test.models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtaildocs", "0010_document_file_hash"),
        ("wagtailimages", "0022_uploadedimage"),
        ("wagtail_localize_test", "0006_auto_20201006_1338"),
    ]

    operations = [
        migrations.AddField(
            model_name="testpage",
            name="test_synchronized_document",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtaildocs.document",
            ),
        ),
        migrations.AddField(
            model_name="testpage",
            name="test_synchronized_image",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="wagtailimages.image",
            ),
        ),
        migrations.AlterField(
            model_name="testgeneratetranslatablefieldspage",
            name="test_streamfield",
            field=wagtail.core.fields.StreamField(
                [
                    ("test_charblock", wagtail.core.blocks.CharBlock(max_length=255)),
                    (
                        "test_textblock",
                        wagtail.core.blocks.TextBlock(label="text block"),
                    ),
                    ("test_emailblock", wagtail.core.blocks.EmailBlock()),
                    ("test_urlblock", wagtail.core.blocks.URLBlock()),
                    ("test_richtextblock", wagtail.core.blocks.RichTextBlock()),
                    ("test_rawhtmlblock", wagtail.core.blocks.RawHTMLBlock()),
                    ("test_blockquoteblock", wagtail.core.blocks.BlockQuoteBlock()),
                    (
                        "test_structblock",
                        wagtail.core.blocks.StructBlock(
                            [
                                ("field_a", wagtail.core.blocks.TextBlock()),
                                ("field_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_listblock",
                        wagtail.core.blocks.ListBlock(wagtail.core.blocks.TextBlock()),
                    ),
                    (
                        "test_nestedstreamblock",
                        wagtail.core.blocks.StreamBlock(
                            [
                                ("block_a", wagtail.core.blocks.TextBlock()),
                                ("block_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_customstructblock",
                        wagtail.core.blocks.StructBlock(
                            [
                                ("field_a", wagtail.core.blocks.TextBlock()),
                                ("field_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_customblockwithoutextractmethod",
                        wagtail_localize.test.models.CustomBlockWithoutExtractMethod(),
                    ),
                    ("test_pagechooserblock", wagtail.core.blocks.PageChooserBlock()),
                    (
                        "test_imagechooserblock",
                        wagtail.images.blocks.ImageChooserBlock(),
                    ),
                    (
                        "test_documentchooserblock",
                        wagtail.documents.blocks.DocumentChooserBlock(),
                    ),
                    (
                        "test_snippetchooserblock",
                        wagtail.snippets.blocks.SnippetChooserBlock(
                            wagtail_localize.test.models.TestSnippet
                        ),
                    ),
                ],
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="testpage",
            name="test_streamfield",
            field=wagtail.core.fields.StreamField(
                [
                    ("test_charblock", wagtail.core.blocks.CharBlock(max_length=255)),
                    (
                        "test_textblock",
                        wagtail.core.blocks.TextBlock(label="text block"),
                    ),
                    ("test_emailblock", wagtail.core.blocks.EmailBlock()),
                    ("test_urlblock", wagtail.core.blocks.URLBlock()),
                    ("test_richtextblock", wagtail.core.blocks.RichTextBlock()),
                    ("test_rawhtmlblock", wagtail.core.blocks.RawHTMLBlock()),
                    ("test_blockquoteblock", wagtail.core.blocks.BlockQuoteBlock()),
                    (
                        "test_structblock",
                        wagtail.core.blocks.StructBlock(
                            [
                                ("field_a", wagtail.core.blocks.TextBlock()),
                                ("field_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_listblock",
                        wagtail.core.blocks.ListBlock(wagtail.core.blocks.TextBlock()),
                    ),
                    (
                        "test_nestedstreamblock",
                        wagtail.core.blocks.StreamBlock(
                            [
                                ("block_a", wagtail.core.blocks.TextBlock()),
                                ("block_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_customstructblock",
                        wagtail.core.blocks.StructBlock(
                            [
                                ("field_a", wagtail.core.blocks.TextBlock()),
                                ("field_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_customblockwithoutextractmethod",
                        wagtail_localize.test.models.CustomBlockWithoutExtractMethod(),
                    ),
                    ("test_pagechooserblock", wagtail.core.blocks.PageChooserBlock()),
                    (
                        "test_imagechooserblock",
                        wagtail.images.blocks.ImageChooserBlock(),
                    ),
                    (
                        "test_documentchooserblock",
                        wagtail.documents.blocks.DocumentChooserBlock(),
                    ),
                    (
                        "test_snippetchooserblock",
                        wagtail.snippets.blocks.SnippetChooserBlock(
                            wagtail_localize.test.models.TestSnippet
                        ),
                    ),
                ],
                blank=True,
            ),
        ),
        migrations.AlterField(
            model_name="testpage",
            name="test_synchronized_streamfield",
            field=wagtail.core.fields.StreamField(
                [
                    ("test_charblock", wagtail.core.blocks.CharBlock(max_length=255)),
                    (
                        "test_textblock",
                        wagtail.core.blocks.TextBlock(label="text block"),
                    ),
                    ("test_emailblock", wagtail.core.blocks.EmailBlock()),
                    ("test_urlblock", wagtail.core.blocks.URLBlock()),
                    ("test_richtextblock", wagtail.core.blocks.RichTextBlock()),
                    ("test_rawhtmlblock", wagtail.core.blocks.RawHTMLBlock()),
                    ("test_blockquoteblock", wagtail.core.blocks.BlockQuoteBlock()),
                    (
                        "test_structblock",
                        wagtail.core.blocks.StructBlock(
                            [
                                ("field_a", wagtail.core.blocks.TextBlock()),
                                ("field_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_listblock",
                        wagtail.core.blocks.ListBlock(wagtail.core.blocks.TextBlock()),
                    ),
                    (
                        "test_nestedstreamblock",
                        wagtail.core.blocks.StreamBlock(
                            [
                                ("block_a", wagtail.core.blocks.TextBlock()),
                                ("block_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_customstructblock",
                        wagtail.core.blocks.StructBlock(
                            [
                                ("field_a", wagtail.core.blocks.TextBlock()),
                                ("field_b", wagtail.core.blocks.TextBlock()),
                            ]
                        ),
                    ),
                    (
                        "test_customblockwithoutextractmethod",
                        wagtail_localize.test.models.CustomBlockWithoutExtractMethod(),
                    ),
                    ("test_pagechooserblock", wagtail.core.blocks.PageChooserBlock()),
                    (
                        "test_imagechooserblock",
                        wagtail.images.blocks.ImageChooserBlock(),
                    ),
                    (
                        "test_documentchooserblock",
                        wagtail.documents.blocks.DocumentChooserBlock(),
                    ),
                    (
                        "test_snippetchooserblock",
                        wagtail.snippets.blocks.SnippetChooserBlock(
                            wagtail_localize.test.models.TestSnippet
                        ),
                    ),
                ],
                blank=True,
            ),
        ),
    ]
